# This script installs AWS App2Container.
# Copyright (c) 2019 Amazon Inc. All rights reserved.

param (
    [switch] $acceptEula = $false
)

$ErrorActionPreference = "Stop"

$script:ApplicationName = "AWSApp2Container"
$script:BinaryName = "app2container.exe"
$script:InstallerZip = Join-Path -Path "$PSScriptRoot" -ChildPath "$script:ApplicationName.zip"
$script:CurrentVersion = "v0.0.0"
$script:PrefixPath = Join-Path -Path "$env:USERPROFILE" -ChildPath "app2container"
$script:InstallPath = Join-Path -Path $script:PrefixPath -ChildPath ("." + $script:ApplicationName)
$script:VersionPath = Join-Path -Path "$script:InstallPath" -ChildPath "$script:ApplicationName"
$script:ApplicationSymlinkPath = Join-Path -Path $script:PrefixPath -ChildPath $script:ApplicationName
$script:PostInstallPath = Join-Path -Path $script:ApplicationSymlinkPath -ChildPath "resources\windows\tools\post-install.ps1"

Function Exit-Installer
{
    Exit
}

Function Assert-AdministratorUserPermissions
{
    $CurrentUserIdentity=[System.Security.Principal.WindowsIdentity]::GetCurrent()
    $CurrentUserPrincipal=New-Object System.Security.Principal.WindowsPrincipal($CurrentUserIdentity)
    Write-Host "Verifying current user permissions..."
    if (-Not $CurrentUserPrincipal.IsInRole("Administrators")) {
        Write-Host "Current user is not in the Administrators group."
        Write-Host "Run the installer as user in the Local Administrators group."
        Exit-Installer
    }
}

Function Get-Current-Version
{
    Try {
        $ApplicationBinaryPath = Join-Path -Path "$script:VersionPath" -ChildPath "bin"
        $ApplicationBinary = Join-Path $ApplicationBinaryPath -ChildPath $script:BinaryName
        $VersionString = Invoke-Command -ScriptBlock {& $ApplicationBinary --version}
        $CurrentVersion = $VersionString.Split(" ")[2]
        Return $CurrentVersion
    } Catch {
        Write-Host "Unable to determine the current version"
        Return "0.0.0"
    }
}

function PSVersionCheck {
    try {
        $psVersion = $PSVersionTable.PSVersion.Major
        if ($psVersion -lt 5) {
            return "PSVersionCheckError"
        }
    } catch {
        return $_
    }
    return "PASS"
}

function Check-Prerequisites {
    $PSVersionCheck = PSVersionCheck
    if ($PSVersionCheck -eq "PSVersionCheckError") {
        throw "Powershell Version 5 or higher not installed. Version 5 or higher is required."
    } elseif ($PSVersionCheck -ne "PASS") {
        throw "Unable to determine the Powershell Version! error = $($PSVersionCheck)"
    }

}

Function Install-App2Container
{
     Write-Host "Checking the current installed A2C version"
     $binaryPath = Join-Path $script:PrefixPath -ChildPath $script:ApplicationName | Join-Path -ChildPath "bin" | Join-Path -ChildPath $script:BinaryName
     if (Test-Path $binaryPath -PathType Leaf) {
        $VersionString = Invoke-Command -ScriptBlock {& $binaryPath --version}
        $installedVersion = $VersionString.Split(" ")[2]
        if ($installedVersion -match '^[0-9]+.[0-9]+(.[0-9]+)?$') {
            Write-Host "Detected installed GA version $installedVersion"
        } else {
            Write-Host "App2Container non GA version $installedVersion is detected. Please remove the $script:PrefixPath folder and try again"
            Exit-Installer
        }
     }

    if (-Not (Test-Path $script:InstallerZip)) {
        Write-Host "Could not find $script:InstallerZip"
        Exit-Installer
    }

    Write-Host "The AWS App2Container tool is licensed as \"AWS Content\" under the terms and conditions of the AWS Customer Agreement, located at https://aws.amazon.com/agreement and the Service Terms, located at https://aws.amazon.com/service-terms. By installing, using or accessing the AWS App2Container tool, you agree to such terms and conditions. The term \"AWS Content\" does not include software and assets distributed under separate license terms (such as code licensed under an open source license)."

    if (-Not $acceptEula) {
        $Choice = Read-Host "Do you accept the terms and conditions above? (y/n)"
        if (($Choice -ne "y") -And ($Choice -ne "Y")) {
            Exit-Installer
        }
    }

    Write-Host "Installing AWS App2Container ..."
    if (Test-Path $script:InstallPath) {
        Remove-Item -Path $script:InstallPath -Force -Recurse -WarningAction Ignore
    }
    New-Item -ItemType Directory -Force -Path $script:InstallPath | Out-Null
    Add-Type -Assembly "System.IO.Compression.FileSystem"
    [IO.Compression.ZipFile]::ExtractToDirectory($script:InstallerZip, $script:InstallPath)
    Start-Sleep -Seconds 3
}

Function Setup-Path
{
    Write-Host "Setting up Path ..."

    $script:CurrentVersion = Get-Current-Version
    Write-Host "Obtained current version: $CurrentVersion"

    # Rename the version folder
    For ($retry = 1; $retry -le 10; $retry++)
    {
        Try
        {
            Rename-Item -LiteralPath "$script:VersionPath" -NewName "v$script:CurrentVersion"
            $NewVersionPath = Join-Path -Path "$script:InstallPath" -ChildPath "v$script:CurrentVersion"
            Write-Host "Version folder rename succeeded"
            Break
        }
        Catch
        {
            Write-Host "Failed version folder rename in attempt $retry. Retrying after 5 second sleep"
            Start-Sleep -Seconds 5
        }
    }

    # Create version folder symlink
    if (Test-Path $script:ApplicationSymlinkPath)
    {
        (Get-Item -Path $script:ApplicationSymlinkPath).Delete()
    }
    New-Item -ItemType SymbolicLink -Path $script:ApplicationSymlinkPath -Target $NewVersionPath | Out-Null

    $ExistingPath = [Environment]::GetEnvironmentVariable("Path", [EnvironmentVariableTarget]::User)
    $BinaryPath = Join-Path -Path "$script:ApplicationSymlinkPath" -ChildPath "bin"
    if (($ExistingPath -eq $null) -Or (-Not $ExistingPath.Contains("$BinaryPath")))
    {
        Write-Host "Adding AWS App2Container to User path"
        $NewPath = $ExistingPath + ";" + $BinaryPath
        [Environment]::SetEnvironmentVariable("Path", $NewPath, [EnvironmentVariableTarget]::User)
        # Add to current session's path
        $env:Path += ";$BinaryPath"
    }
}

Function Installer
{
    Assert-AdministratorUserPermissions
    Check-Prerequisites
    Install-App2Container
    Setup-Path
    . "$script:PostInstallPath"

    Write-Host
    Write-Host "Installation of AWS App2Container completed successfully!"
    Write-Host "You are currently running version $script:CurrentVersion."
    Write-Host "To get started, run 'app2container init'"
    Write-Host "AWS App2Container was installed in $script:ApplicationSymlinkPath."
}

Installer
# SIG # Begin signature block
# MIIuegYJKoZIhvcNAQcCoIIuazCCLmcCAQExDzANBglghkgBZQMEAgEFADB5Bgor
# BgEEAYI3AgEEoGswaTA0BgorBgEEAYI3AgEeMCYCAwEAAAQQH8w7YFlLCE63JNLG
# KX7zUQIBAAIBAAIBAAIBAAIBADAxMA0GCWCGSAFlAwQCAQUABCAJNlV/dMlgOlLl
# 1oqVyY8O4vVSkPOPmPPfMvPFTbVaBaCCE+YwggXAMIIEqKADAgECAhAP0bvKeWvX
# +N1MguEKmpYxMA0GCSqGSIb3DQEBCwUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNV
# BAMTIkRpZ2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMjIwMTEz
# MDAwMDAwWhcNMzExMTA5MjM1OTU5WjBiMQswCQYDVQQGEwJVUzEVMBMGA1UEChMM
# RGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSEwHwYDVQQD
# ExhEaWdpQ2VydCBUcnVzdGVkIFJvb3QgRzQwggIiMA0GCSqGSIb3DQEBAQUAA4IC
# DwAwggIKAoICAQC/5pBzaN675F1KPDAiMGkz7MKnJS7JIT3yithZwuEppz1Yq3aa
# za57G4QNxDAf8xukOBbrVsaXbR2rsnnyyhHS5F/WBTxSD1Ifxp4VpX6+n6lXFllV
# cq9ok3DCsrp1mWpzMpTREEQQLt+C8weE5nQ7bXHiLQwb7iDVySAdYyktzuxeTsiT
# +CFhmzTrBcZe7FsavOvJz82sNEBfsXpm7nfISKhmV1efVFiODCu3T6cw2Vbuyntd
# 463JT17lNecxy9qTXtyOj4DatpGYQJB5w3jHtrHEtWoYOAMQjdjUN6QuBX2I9YI+
# EJFwq1WCQTLX2wRzKm6RAXwhTNS8rhsDdV14Ztk6MUSaM0C/CNdaSaTC5qmgZ92k
# J7yhTzm1EVgX9yRcRo9k98FpiHaYdj1ZXUJ2h4mXaXpI8OCiEhtmmnTK3kse5w5j
# rubU75KSOp493ADkRSWJtppEGSt+wJS00mFt6zPZxd9LBADMfRyVw4/3IbKyEbe7
# f/LVjHAsQWCqsWMYRJUadmJ+9oCw++hkpjPRiQfhvbfmQ6QYuKZ3AeEPlAwhHbJU
# KSWJbOUOUlFHdL4mrLZBdd56rF+NP8m800ERElvlEFDrMcXKchYiCd98THU/Y+wh
# X8QgUWtvsauGi0/C1kVfnSD8oR7FwI+isX4KJpn15GkvmB0t9dmpsh3lGwIDAQAB
# o4IBZjCCAWIwDwYDVR0TAQH/BAUwAwEB/zAdBgNVHQ4EFgQU7NfjgtJxXWRM3y5n
# P+e6mK4cD08wHwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8MwDgYDVR0P
# AQH/BAQDAgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMH8GCCsGAQUFBwEBBHMwcTAk
# BggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEkGCCsGAQUFBzAC
# hj1odHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRIaWdoQXNzdXJh
# bmNlRVZSb290Q0EuY3J0MEsGA1UdHwREMEIwQKA+oDyGOmh0dHA6Ly9jcmwzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEhpZ2hBc3N1cmFuY2VFVlJvb3RDQS5jcmwwHAYD
# VR0gBBUwEzAHBgVngQwBAzAIBgZngQwBBAEwDQYJKoZIhvcNAQELBQADggEBAEHx
# qRH0DxNHecllao3A7pgEpMbjDPKisedfYk/ak1k2zfIe4R7sD+EbP5HU5A/C5pg0
# /xkPZigfT2IxpCrhKhO61z7H0ZL+q93fqpgzRh9Onr3g7QdG64AupP2uU7SkwaT1
# IY1rzAGt9Rnu15ClMlIr28xzDxj4+87eg3Gn77tRWwR2L62t0+od/P1Tk+WMieNg
# GbngLyOOLFxJy34riDkruQZhiPOuAnZ2dMFkkbiJUZflhX0901emWG4f7vtpYeJa
# 3Cgh6GO6Ps9W7Zrk9wXqyvPsEt84zdp7PiuTUy9cUQBY3pBIowrHC/Q7bVUx8ALM
# R3eWUaNetbxcyEMRoacwggawMIIEmKADAgECAhAIrUCyYNKcTJ9ezam9k67ZMA0G
# CSqGSIb3DQEBDAUAMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdpQ2VydCBJ
# bmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNVBAMTGERpZ2lDZXJ0
# IFRydXN0ZWQgUm9vdCBHNDAeFw0yMTA0MjkwMDAwMDBaFw0zNjA0MjgyMzU5NTla
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQDVtC9C
# 0CiteLdd1TlZG7GIQvUzjOs9gZdwxbvEhSYwn6SOaNhc9es0JAfhS0/TeEP0F9ce
# 2vnS1WcaUk8OoVf8iJnBkcyBAz5NcCRks43iCH00fUyAVxJrQ5qZ8sU7H/Lvy0da
# E6ZMswEgJfMQ04uy+wjwiuCdCcBlp/qYgEk1hz1RGeiQIXhFLqGfLOEYwhrMxe6T
# SXBCMo/7xuoc82VokaJNTIIRSFJo3hC9FFdd6BgTZcV/sk+FLEikVoQ11vkunKoA
# FdE3/hoGlMJ8yOobMubKwvSnowMOdKWvObarYBLj6Na59zHh3K3kGKDYwSNHR7Oh
# D26jq22YBoMbt2pnLdK9RBqSEIGPsDsJ18ebMlrC/2pgVItJwZPt4bRc4G/rJvmM
# 1bL5OBDm6s6R9b7T+2+TYTRcvJNFKIM2KmYoX7BzzosmJQayg9Rc9hUZTO1i4F4z
# 8ujo7AqnsAMrkbI2eb73rQgedaZlzLvjSFDzd5Ea/ttQokbIYViY9XwCFjyDKK05
# huzUtw1T0PhH5nUwjewwk3YUpltLXXRhTT8SkXbev1jLchApQfDVxW0mdmgRQRNY
# mtwmKwH0iU1Z23jPgUo+QEdfyYFQc4UQIyFZYIpkVMHMIRroOBl8ZhzNeDhFMJlP
# /2NPTLuqDQhTQXxYPUez+rbsjDIJAsxsPAxWEQIDAQABo4IBWTCCAVUwEgYDVR0T
# AQH/BAgwBgEB/wIBADAdBgNVHQ4EFgQUaDfg67Y7+F8Rhvv+YXsIiGX0TkIwHwYD
# VR0jBBgwFoAU7NfjgtJxXWRM3y5nP+e6mK4cD08wDgYDVR0PAQH/BAQDAgGGMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMHcGCCsGAQUFBwEBBGswaTAkBggrBgEFBQcwAYYY
# aHR0cDovL29jc3AuZGlnaWNlcnQuY29tMEEGCCsGAQUFBzAChjVodHRwOi8vY2Fj
# ZXJ0cy5kaWdpY2VydC5jb20vRGlnaUNlcnRUcnVzdGVkUm9vdEc0LmNydDBDBgNV
# HR8EPDA6MDigNqA0hjJodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNlcnRU
# cnVzdGVkUm9vdEc0LmNybDAcBgNVHSAEFTATMAcGBWeBDAEDMAgGBmeBDAEEATAN
# BgkqhkiG9w0BAQwFAAOCAgEAOiNEPY0Idu6PvDqZ01bgAhql+Eg08yy25nRm95Ry
# sQDKr2wwJxMSnpBEn0v9nqN8JtU3vDpdSG2V1T9J9Ce7FoFFUP2cvbaF4HZ+N3HL
# IvdaqpDP9ZNq4+sg0dVQeYiaiorBtr2hSBh+3NiAGhEZGM1hmYFW9snjdufE5Btf
# Q/g+lP92OT2e1JnPSt0o618moZVYSNUa/tcnP/2Q0XaG3RywYFzzDaju4ImhvTnh
# OE7abrs2nfvlIVNaw8rpavGiPttDuDPITzgUkpn13c5UbdldAhQfQDN8A+KVssIh
# dXNSy0bYxDQcoqVLjc1vdjcshT8azibpGL6QB7BDf5WIIIJw8MzK7/0pNVwfiThV
# 9zeKiwmhywvpMRr/LhlcOXHhvpynCgbWJme3kuZOX956rEnPLqR0kq3bPKSchh/j
# wVYbKyP/j7XqiHtwa+aguv06P0WmxOgWkVKLQcBIhEuWTatEQOON8BUozu3xGFYH
# Ki8QxAwIZDwzj64ojDzLj4gLDb879M4ee47vtevLt/B3E+bnKD+sEq6lLyJsQfmC
# XBVmzGwOysWGw/YmMwwHS6DTBwJqakAwSEs0qFEgu60bhQjiWQ1tygVQK+pKHJ6l
# /aCnHwZ05/LWUpD9r4VIIflXO7ScA+2GRfS0YW6/aOImYIbqyK+p/pQd52MbOoZW
# eE4wggdqMIIFUqADAgECAhAH5QNwxSTqWWdwQMQBH6CZMA0GCSqGSIb3DQEBCwUA
# MGkxCzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UE
# AxM4RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEz
# ODQgMjAyMSBDQTEwHhcNMjMwOTIxMDAwMDAwWhcNMjQwOTIwMjM1OTU5WjCB8jET
# MBEGCysGAQQBgjc8AgEDEwJVUzEZMBcGCysGAQQBgjc8AgECEwhEZWxhd2FyZTEd
# MBsGA1UEDwwUUHJpdmF0ZSBPcmdhbml6YXRpb24xEDAOBgNVBAUTBzQxNTI5NTQx
# CzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdTZWF0
# dGxlMSIwIAYDVQQKExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMuMRMwEQYDVQQL
# EwpBbWF6b24gRUMyMSIwIAYDVQQDExlBbWF6b24gV2ViIFNlcnZpY2VzLCBJbmMu
# MIIBojANBgkqhkiG9w0BAQEFAAOCAY8AMIIBigKCAYEAyztIbHTQkOxMy15gO5qs
# l2ogfybSwPrJZpJZnJ+LLeF6oxijTxC9fQG60GInL4DdQNgiLWWWZLvH45GZ7pho
# Z46Bj8cLT1tq2iYsxoycxtwltW+BhKCVSCdpYHA1Qmw5OpiwPfTtjrAyBHdGuRPd
# M6SzS9E8KeA0xANG7SJRLwFFOYBV6aBr0KGTJsVqRQRfJgUljh7Ft1rxjETKcq3Z
# RxuVZcqp5DaZHdTBtVoQFxUHO16hbRZt/9J3+oOkNZ9NpnyQzla0ei3UzTGWcUjO
# pTIYQ5yfqJrM3tbfNBYhdifMvj+3BzgRwVDOh1rhXM19XjFj11dvUfky1SuwzWnq
# Y0nppt8rQyhr5VgjMEM5SvBlOCnei9PPXOB/3OrN2b/8PA+B3AsOQolsW1PgnHPR
# daBcteb9WlQN1tdzPF48KfY3x549+i0weQqkNHP/aLcUGxYPpuJpDLAz0uD/E9W+
# jbaGXskUMgY2PNCVwClrTnpnUMK0PavsfV1WVZPhWuxVAgMBAAGjggICMIIB/jAf
# BgNVHSMEGDAWgBRoN+Drtjv4XxGG+/5hewiIZfROQjAdBgNVHQ4EFgQUL0m+3RGA
# 0eThZQtWdO36iJ5VPZswPQYDVR0gBDYwNDAyBgVngQwBAzApMCcGCCsGAQUFBwIB
# FhtodHRwOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwDgYDVR0PAQH/BAQDAgeAMBMG
# A1UdJQQMMAoGCCsGAQUFBwMDMIG1BgNVHR8Ega0wgaowU6BRoE+GTWh0dHA6Ly9j
# cmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRHNENvZGVTaWduaW5nUlNB
# NDA5NlNIQTM4NDIwMjFDQTEuY3JsMFOgUaBPhk1odHRwOi8vY3JsNC5kaWdpY2Vy
# dC5jb20vRGlnaUNlcnRUcnVzdGVkRzRDb2RlU2lnbmluZ1JTQTQwOTZTSEEzODQy
# MDIxQ0ExLmNybDCBlAYIKwYBBQUHAQEEgYcwgYQwJAYIKwYBBQUHMAGGGGh0dHA6
# Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBcBggrBgEFBQcwAoZQaHR0cDovL2NhY2VydHMu
# ZGlnaWNlcnQuY29tL0RpZ2lDZXJ0VHJ1c3RlZEc0Q29kZVNpZ25pbmdSU0E0MDk2
# U0hBMzg0MjAyMUNBMS5jcnQwCQYDVR0TBAIwADANBgkqhkiG9w0BAQsFAAOCAgEA
# T+LfKWm0mxDaWea9AC1Zr46CtIMq2ts0B4FCJG9B7FibjGHPOfFesSSLe0xwVr4g
# JHUzygEgt+Jb1M6vuSQAebV4ux25T0C/6jfkgvDxT99IZYykDSCG1xl4q08sxmp1
# mJUpS7TTKWXSXoTqAi0cw9LvF57G5yb7C4OXA2Kzqc3IAo2R3YKWdKi58Mh9cgrP
# n0sxNqak1vdFc2GFRZb5OB8siapl/SLzjXs2Tlzk2gVzMLXcH1IZtaYlODtu+m/v
# q35E0xXlq6PJdG4pjRZj07zh1gWlNDhaCKt37UcsHSxElhtOXaJuc2pOi+m7LXLX
# Zo1T8MujDJbudDUnt4Okr3ABBVBNfPfideVFFSBXyn69L1JteHkTVQ8UMZcHpykO
# boeYd/SCIHPzgCluqRIYMeGUppcSU3tKw30mri7Ky2xTYPOiqu6Bnatf2dM79F9I
# s+NQYMZMpT9zxpDKy3sLsv0fHMQ2ck5pwlYrtFLecP14O2SOyOdme7apJnXqYHse
# 0iGegkFPzccRQcvl6ov3mGqndlhVK4SitJ89K7tjSbnZoJ7VKHLLIohUkrq5lNt2
# EbjYO3KGgG8Aoo+YbWlNkekFHi+avdVC5igeXES/7vPH5cNRawGL2IvJf71Sw+Db
# eg9487ZMdhy0zUf71Uo8l7qK9LDJKFz+FKSTsjNS2hcxghnqMIIZ5gIBATB9MGkx
# CzAJBgNVBAYTAlVTMRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjFBMD8GA1UEAxM4
# RGlnaUNlcnQgVHJ1c3RlZCBHNCBDb2RlIFNpZ25pbmcgUlNBNDA5NiBTSEEzODQg
# MjAyMSBDQTECEAflA3DFJOpZZ3BAxAEfoJkwDQYJYIZIAWUDBAIBBQCgfDAQBgor
# BgEEAYI3AgEMMQIwADAZBgkqhkiG9w0BCQMxDAYKKwYBBAGCNwIBBDAcBgorBgEE
# AYI3AgELMQ4wDAYKKwYBBAGCNwIBFTAvBgkqhkiG9w0BCQQxIgQg0LB91VecPKF5
# l9MTQ4uFeiiogx9vDn2EWFEZi4LyGfgwDQYJKoZIhvcNAQEBBQAEggGApYAeVsdH
# 3DwCS4z4v+E9ML94lpR0N5sAL9tGuRqCFN4HMHaVA/C0M+X2zE1GCt6GRaA6UdJd
# KC1+5mkHGWf4oHCimAelzoS8cCp+L32FyDexDVLeutK6yQM2IkNiqQJ3j+5PqJt7
# 8IRui5CAxQicKCEmVOZzR1QxYeXsV/jsS+qmFAscqOfoFY0N+L4mJIBguYb0LJnI
# 6AD9GzjBGgCHtR2T7b7W8nGJb/ZCsiiT8BNKZIHMx6Ra7/llMUsJTd0AtF/ZCOt3
# UM3KrCA0Gx4EvC7/AKegMuB8tCOGDt0ug5wO1KxVAvKgdQxS9jBTn+SYvI9M2Htt
# IztsJszOcvzhttZjC6W1cWXTOLCc0s4/6KN0mHdRuohs1tKwg4RBxDcitOcWCr/C
# HKwp2JMbR5xLDoCVWm3Lt1xOxR6Av9E2OEr8/5vCS8zHZK3/tlIBeMGHSi72Qn4A
# slbkFoSQaEg8OAGm2gGcdxPDvfns20VchL3UY+DNrHhvCe47gSuux+QLoYIXQDCC
# FzwGCisGAQQBgjcDAwExghcsMIIXKAYJKoZIhvcNAQcCoIIXGTCCFxUCAQMxDzAN
# BglghkgBZQMEAgEFADB4BgsqhkiG9w0BCRABBKBpBGcwZQIBAQYJYIZIAYb9bAcB
# MDEwDQYJYIZIAWUDBAIBBQAEIN1wCIAGW712tkp4SYtVN1PUdaKrYFKLjwgqvLoJ
# bVYeAhEAhi1J2ar4ROcfqCKAHyKK+xgPMjAyNDAzMDkwMzM4NTlaoIITCTCCBsIw
# ggSqoAMCAQICEAVEr/OUnQg5pr/bP1/lYRYwDQYJKoZIhvcNAQELBQAwYzELMAkG
# A1UEBhMCVVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdp
# Q2VydCBUcnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQTAe
# Fw0yMzA3MTQwMDAwMDBaFw0zNDEwMTMyMzU5NTlaMEgxCzAJBgNVBAYTAlVTMRcw
# FQYDVQQKEw5EaWdpQ2VydCwgSW5jLjEgMB4GA1UEAxMXRGlnaUNlcnQgVGltZXN0
# YW1wIDIwMjMwggIiMA0GCSqGSIb3DQEBAQUAA4ICDwAwggIKAoICAQCjU0WHHYOO
# W6w+VLMj4M+f1+XS512hDgncL0ijl3o7Kpxn3GIVWMGpkxGnzaqyat0QKYoeYmNp
# 01icNXG/OpfrlFCPHCDqx5o7L5Zm42nnaf5bw9YrIBzBl5S0pVCB8s/LB6YwaMqD
# Qtr8fwkklKSCGtpqutg7yl3eGRiF+0XqDWFsnf5xXsQGmjzwxS55DxtmUuPI1j5f
# 2kPThPXQx/ZILV5FdZZ1/t0QoRuDwbjmUpW1R9d4KTlr4HhZl+NEK0rVlc7vCBfq
# gmRN/yPjyobutKQhZHDr1eWg2mOzLukF7qr2JPUdvJscsrdf3/Dudn0xmWVHVZ1K
# JC+sK5e+n+T9e3M+Mu5SNPvUu+vUoCw0m+PebmQZBzcBkQ8ctVHNqkxmg4hoYru8
# QRt4GW3k2Q/gWEH72LEs4VGvtK0VBhTqYggT02kefGRNnQ/fztFejKqrUBXJs8q8
# 18Q7aESjpTtC/XN97t0K/3k0EH6mXApYTAA+hWl1x4Nk1nXNjxJ2VqUk+tfEayG6
# 6B80mC866msBsPf7Kobse1I4qZgJoXGybHGvPrhvltXhEBP+YUcKjP7wtsfVx95s
# JPC/QoLKoHE9nJKTBLRpcCcNT7e1NtHJXwikcKPsCvERLmTgyyIryvEoEyFJUX4G
# ZtM7vvrrkTjYUQfKlLfiUKHzOtOKg8tAewIDAQABo4IBizCCAYcwDgYDVR0PAQH/
# BAQDAgeAMAwGA1UdEwEB/wQCMAAwFgYDVR0lAQH/BAwwCgYIKwYBBQUHAwgwIAYD
# VR0gBBkwFzAIBgZngQwBBAIwCwYJYIZIAYb9bAcBMB8GA1UdIwQYMBaAFLoW2W1N
# hS9zKXaaL3WMaiCPnshvMB0GA1UdDgQWBBSltu8T5+/N0GSh1VapZTGj3tXjSTBa
# BgNVHR8EUzBRME+gTaBLhklodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vRGlnaUNl
# cnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3JsMIGQBggr
# BgEFBQcBAQSBgzCBgDAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuZGlnaWNlcnQu
# Y29tMFgGCCsGAQUFBzAChkxodHRwOi8vY2FjZXJ0cy5kaWdpY2VydC5jb20vRGln
# aUNlcnRUcnVzdGVkRzRSU0E0MDk2U0hBMjU2VGltZVN0YW1waW5nQ0EuY3J0MA0G
# CSqGSIb3DQEBCwUAA4ICAQCBGtbeoKm1mBe8cI1PijxonNgl/8ss5M3qXSKS7Iwi
# Aqm4z4Co2efjxe0mgopxLxjdTrbebNfhYJwr7e09SI64a7p8Xb3CYTdoSXej65Cq
# EtcnhfOOHpLawkA4n13IoC4leCWdKgV6hCmYtld5j9smViuw86e9NwzYmHZPVrlS
# wradOKmB521BXIxp0bkrxMZ7z5z6eOKTGnaiaXXTUOREEr4gDZ6pRND45Ul3CFoh
# xbTPmJUaVLq5vMFpGbrPFvKDNzRusEEm3d5al08zjdSNd311RaGlWCZqA0Xe2VC1
# UIyvVr1MxeFGxSjTredDAHDezJieGYkD6tSRN+9NUvPJYCHEVkft2hFLjDLDiOZY
# 4rbbPvlfsELWj+MXkdGqwFXjhr+sJyxB0JozSqg21Llyln6XeThIX8rC3D0y33XW
# Nmdaifj2p8flTzU8AL2+nCpseQHc2kTmOt44OwdeOVj0fHMxVaCAEcsUDH6uvP6k
# 63llqmjWIso765qCNVcoFstp8jKastLYOrixRoZruhf9xHdsFWyuq69zOuhJRrfV
# f8y2OMDY7Bz1tqG4QyzfTkx9HmhwwHcK1ALgXGC7KP845VJa1qwXIiNO9OzTF/tQ
# a/8Hdx9xl0RBybhG02wyfFgvZ0dl5Rtztpn5aywGRu9BHvDwX+Db2a2QgESvgBBB
# ijCCBq4wggSWoAMCAQICEAc2N7ckVHzYR6z9KGYqXlswDQYJKoZIhvcNAQELBQAw
# YjELMAkGA1UEBhMCVVMxFTATBgNVBAoTDERpZ2lDZXJ0IEluYzEZMBcGA1UECxMQ
# d3d3LmRpZ2ljZXJ0LmNvbTEhMB8GA1UEAxMYRGlnaUNlcnQgVHJ1c3RlZCBSb290
# IEc0MB4XDTIyMDMyMzAwMDAwMFoXDTM3MDMyMjIzNTk1OVowYzELMAkGA1UEBhMC
# VVMxFzAVBgNVBAoTDkRpZ2lDZXJ0LCBJbmMuMTswOQYDVQQDEzJEaWdpQ2VydCBU
# cnVzdGVkIEc0IFJTQTQwOTYgU0hBMjU2IFRpbWVTdGFtcGluZyBDQTCCAiIwDQYJ
# KoZIhvcNAQEBBQADggIPADCCAgoCggIBAMaGNQZJs8E9cklRVcclA8TykTepl1Gh
# 1tKD0Z5Mom2gsMyD+Vr2EaFEFUJfpIjzaPp985yJC3+dH54PMx9QEwsmc5Zt+Feo
# An39Q7SE2hHxc7Gz7iuAhIoiGN/r2j3EF3+rGSs+QtxnjupRPfDWVtTnKC3r07G1
# decfBmWNlCnT2exp39mQh0YAe9tEQYncfGpXevA3eZ9drMvohGS0UvJ2R/dhgxnd
# X7RUCyFobjchu0CsX7LeSn3O9TkSZ+8OpWNs5KbFHc02DVzV5huowWR0QKfAcsW6
# Th+xtVhNef7Xj3OTrCw54qVI1vCwMROpVymWJy71h6aPTnYVVSZwmCZ/oBpHIEPj
# Q2OAe3VuJyWQmDo4EbP29p7mO1vsgd4iFNmCKseSv6De4z6ic/rnH1pslPJSlREr
# WHRAKKtzQ87fSqEcazjFKfPKqpZzQmiftkaznTqj1QPgv/CiPMpC3BhIfxQ0z9JM
# q++bPf4OuGQq+nUoJEHtQr8FnGZJUlD0UfM2SU2LINIsVzV5K6jzRWC8I41Y99xh
# 3pP+OcD5sjClTNfpmEpYPtMDiP6zj9NeS3YSUZPJjAw7W4oiqMEmCPkUEBIDfV8j
# u2TjY+Cm4T72wnSyPx4JduyrXUZ14mCjWAkBKAAOhFTuzuldyF4wEr1GnrXTdrnS
# DmuZDNIztM2xAgMBAAGjggFdMIIBWTASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1Ud
# DgQWBBS6FtltTYUvcyl2mi91jGogj57IbzAfBgNVHSMEGDAWgBTs1+OC0nFdZEzf
# Lmc/57qYrhwPTzAOBgNVHQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYBBQUHAwgw
# dwYIKwYBBQUHAQEEazBpMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5kaWdpY2Vy
# dC5jb20wQQYIKwYBBQUHMAKGNWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0LmNvbS9E
# aWdpQ2VydFRydXN0ZWRSb290RzQuY3J0MEMGA1UdHwQ8MDowOKA2oDSGMmh0dHA6
# Ly9jcmwzLmRpZ2ljZXJ0LmNvbS9EaWdpQ2VydFRydXN0ZWRSb290RzQuY3JsMCAG
# A1UdIAQZMBcwCAYGZ4EMAQQCMAsGCWCGSAGG/WwHATANBgkqhkiG9w0BAQsFAAOC
# AgEAfVmOwJO2b5ipRCIBfmbW2CFC4bAYLhBNE88wU86/GPvHUF3iSyn7cIoNqilp
# /GnBzx0H6T5gyNgL5Vxb122H+oQgJTQxZ822EpZvxFBMYh0MCIKoFr2pVs8Vc40B
# IiXOlWk/R3f7cnQU1/+rT4osequFzUNf7WC2qk+RZp4snuCKrOX9jLxkJodskr2d
# fNBwCnzvqLx1T7pa96kQsl3p/yhUifDVinF2ZdrM8HKjI/rAJ4JErpknG6skHibB
# t94q6/aesXmZgaNWhqsKRcnfxI2g55j7+6adcq/Ex8HBanHZxhOACcS2n82HhyS7
# T6NJuXdmkfFynOlLAlKnN36TU6w7HQhJD5TNOXrd/yVjmScsPT9rp/Fmw0HNT7ZA
# myEhQNC3EyTN3B14OuSereU0cZLXJmvkOHOrpgFPvT87eK1MrfvElXvtCl8zOYdB
# eHo46Zzh3SP9HSjTx/no8Zhf+yvYfvJGnXUsHicsJttvFXseGYs2uJPU5vIXmVnK
# cPA3v5gA3yAWTyf7YGcWoWa63VXAOimGsJigK+2VQbc61RWYMbRiCQ8KvYHZE/6/
# pNHzV9m8BPqC3jLfBInwAM1dwvnQI38AC+R2AibZ8GV2QqYphwlHK+Z/GqSFD/yY
# lvZVVCsfgPrA8g4r5db7qS9EFUrnEw4d2zc4GqEr9u3WfPwwggWNMIIEdaADAgEC
# AhAOmxiO+dAt5+/bUOIIQBhaMA0GCSqGSIb3DQEBDAUAMGUxCzAJBgNVBAYTAlVT
# MRUwEwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5j
# b20xJDAiBgNVBAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0yMjA4
# MDEwMDAwMDBaFw0zMTExMDkyMzU5NTlaMGIxCzAJBgNVBAYTAlVTMRUwEwYDVQQK
# EwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xITAfBgNV
# BAMTGERpZ2lDZXJ0IFRydXN0ZWQgUm9vdCBHNDCCAiIwDQYJKoZIhvcNAQEBBQAD
# ggIPADCCAgoCggIBAL/mkHNo3rvkXUo8MCIwaTPswqclLskhPfKK2FnC4SmnPVir
# dprNrnsbhA3EMB/zG6Q4FutWxpdtHauyefLKEdLkX9YFPFIPUh/GnhWlfr6fqVcW
# WVVyr2iTcMKyunWZanMylNEQRBAu34LzB4TmdDttceItDBvuINXJIB1jKS3O7F5O
# yJP4IWGbNOsFxl7sWxq868nPzaw0QF+xembud8hIqGZXV59UWI4MK7dPpzDZVu7K
# e13jrclPXuU15zHL2pNe3I6PgNq2kZhAkHnDeMe2scS1ahg4AxCN2NQ3pC4FfYj1
# gj4QkXCrVYJBMtfbBHMqbpEBfCFM1LyuGwN1XXhm2ToxRJozQL8I11pJpMLmqaBn
# 3aQnvKFPObURWBf3JFxGj2T3wWmIdph2PVldQnaHiZdpekjw4KISG2aadMreSx7n
# DmOu5tTvkpI6nj3cAORFJYm2mkQZK37AlLTSYW3rM9nF30sEAMx9HJXDj/chsrIR
# t7t/8tWMcCxBYKqxYxhElRp2Yn72gLD76GSmM9GJB+G9t+ZDpBi4pncB4Q+UDCEd
# slQpJYls5Q5SUUd0viastkF13nqsX40/ybzTQRESW+UQUOsxxcpyFiIJ33xMdT9j
# 7CFfxCBRa2+xq4aLT8LWRV+dIPyhHsXAj6KxfgommfXkaS+YHS312amyHeUbAgMB
# AAGjggE6MIIBNjAPBgNVHRMBAf8EBTADAQH/MB0GA1UdDgQWBBTs1+OC0nFdZEzf
# Lmc/57qYrhwPTzAfBgNVHSMEGDAWgBRF66Kv9JLLgjEtUYunpyGd823IDzAOBgNV
# HQ8BAf8EBAMCAYYweQYIKwYBBQUHAQEEbTBrMCQGCCsGAQUFBzABhhhodHRwOi8v
# b2NzcC5kaWdpY2VydC5jb20wQwYIKwYBBQUHMAKGN2h0dHA6Ly9jYWNlcnRzLmRp
# Z2ljZXJ0LmNvbS9EaWdpQ2VydEFzc3VyZWRJRFJvb3RDQS5jcnQwRQYDVR0fBD4w
# PDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJl
# ZElEUm9vdENBLmNybDARBgNVHSAECjAIMAYGBFUdIAAwDQYJKoZIhvcNAQEMBQAD
# ggEBAHCgv0NcVec4X6CjdBs9thbX979XB72arKGHLOyFXqkauyL4hxppVCLtpIh3
# bb0aFPQTSnovLbc47/T/gLn4offyct4kvFIDyE7QKt76LVbP+fT3rDB6mouyXtTP
# 0UNEm0Mh65ZyoUi0mcudT6cGAxN3J0TU53/oWajwvy8LpunyNDzs9wPHh6jSTEAZ
# NUZqaVSwuKFWjuyk1T3osdz9HNj0d1pcVIxv76FQPfx2CWiEn2/K2yCNNWAcAgPL
# ILCsWKAOQGPFmCLBsln1VWvPJ6tsds5vIy30fnFqI2si/xK4VC0nftg62fC2h5b9
# W9FcrBjDTZ9ztwGpn1eqXijiuZQxggN2MIIDcgIBATB3MGMxCzAJBgNVBAYTAlVT
# MRcwFQYDVQQKEw5EaWdpQ2VydCwgSW5jLjE7MDkGA1UEAxMyRGlnaUNlcnQgVHJ1
# c3RlZCBHNCBSU0E0MDk2IFNIQTI1NiBUaW1lU3RhbXBpbmcgQ0ECEAVEr/OUnQg5
# pr/bP1/lYRYwDQYJYIZIAWUDBAIBBQCggdEwGgYJKoZIhvcNAQkDMQ0GCyqGSIb3
# DQEJEAEEMBwGCSqGSIb3DQEJBTEPFw0yNDAzMDkwMzM4NTlaMCsGCyqGSIb3DQEJ
# EAIMMRwwGjAYMBYEFGbwKzLCwskPgl3OqorJxk8ZnM9AMC8GCSqGSIb3DQEJBDEi
# BCCvMjd0eXnifHmMp+Bvqdl+5pyzAIOL3N3F7AqUzTMV1zA3BgsqhkiG9w0BCRAC
# LzEoMCYwJDAiBCDS9uRt7XQizNHUQFdoQTZvgoraVZquMxavTRqa1Ax4KDANBgkq
# hkiG9w0BAQEFAASCAgAgb+IaxnPCNmz7PwNION0ZRW+509OMOJjfgtzVa+h4HyuZ
# dri8b0ggEzP040i0DfQI6LKtMNGe8b3hjvZPZexuTWNjeU5bcWioOvIlSJ0OcMbK
# ZFrNZdizoiTHJJs2R2M7C4RZHdik7w0tHe2JFxv4dxTc7efwzhJQfcGST+DhSI0W
# nj+H5ZVfoANTGRz4qEBWmc35Ios+3fbOB1EX5faeRfgNRRQbBg0YCO+6BGMYs6r1
# 9+EXMAjfukYZpQDR6/cPPIz1WZ22jO932xCuyOWarDbn0xemQXZE+9jl6qoL9JaY
# sjI2p3hUo8Q4J9t08r/oM4qAx3im+Ea9R0yP1uaSij04qvwkOzMXNZiRQ0qnC5kH
# /mi4VTOehhG+sRZ6l+IN8xnvEsv5jsoxO5c63ASeUHrzkEl0JK4YZWNPTLoSNcPE
# m6+0xswFTzjU0p63nUuFcLaulTpizikunkvXPftbnnyXf0zBroY52xEZgf38NALY
# 55B10dtXJiCj64PTkYfhJgOIqKi9nK+9ouMwjmkHn4TWySZYb62DAVjwOe44eonP
# VYqNSc1Ns5/Q0YcZgQa9A/fm1XNJXyaWypugkZQFtAYBH7Zip3l33ohUm2Wpq/7w
# tPnwcPpghD+Xh5OF4JU+1/crFb7csA4lnRWCP5QB4BCSYsZ5UC4SBGUOedTDYA==
# SIG # End signature block
